# -*- coding: utf-8 -*-

"""Main module."""

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as mdates

from collections import OrderedDict

def gantt(task=None, start=None, finish=None, **kwargs):
    """ Plot a gantt chart.
    """

    if 'task_type' in kwargs:
        task_type = kwargs['task_type']
    if 'color' in kwargs:
        color = kwargs['color']
    
    USES_DATES = False    
    if np.issubdtype(start.dtype, np.datetime64):
        start = mdates.date2num(start)
        USES_DATES = True
    if np.issubdtype(finish.dtype, np.datetime64):
        finish = mdates.date2num(finish)

    delta = finish-start

    ax = plt.gca()

    labels = []
    
    # TODO: refactor?    
    encoded_tasks = OrderedDict()
    k = 0
    for n in task:
        if not n in encoded_tasks:
            encoded_tasks[n] = k
            k+=1
            
    labels = list(encoded_tasks)
    for i, task in enumerate(task):
        j=encoded_tasks[task]
        if color:
            c = color[task_type[i]] 
        else:
            c = None
        ax.broken_barh([(start[i], delta[i])], (j-0.4,0.8), color=c)

    # Set yticks
    ax.set_yticks(range(len(labels)))
    ax.set_yticklabels(labels) 

    # Set xticks formatting
    # TODO: use matplotlib.dates.AutoDateFormatter
    if USES_DATES:
        ax.xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
        fig = plt.gcf()
        fig.autofmt_xdate()

    ax.invert_yaxis()



# -*- coding: utf-8 -*-

import pandas as pd
import numpy as np
from functools import reduce



def get_starts(arr):
    bounded = np.hstack(([0], arr, [0]))
    diffs = np.diff(bounded)
    starts, = np.where(diffs)
    return starts

def with_full_nans(arr):
    arr = arr.copy()
    arr[np.isnan(arr)] = 0
    return arr

def get_all_starts(arr):
    normal_starts = get_starts(with_full_nans(arr))
    nan_starts = get_starts(np.isnan(arr))
    return np.union1d(normal_starts, nan_starts)


def binary_encode_pd_series(series):
#     this is giving end exclusive, which seems fair
    all_starts = get_all_starts(series.values)
    starts = all_starts[:-1]
    #     the final value of end will not exist in the original series
    ends = all_starts[1:]
    return starts, ends

def series_to_gantt_format(series, index, starts_ends):
    categories = series.iloc[starts_ends[0]]
    starts = index.iloc[starts_ends[0]]
    ends = index.iloc[starts_ends[1]]
    #ends = pd.concat([index.iloc[starts_ends[1][:-1]], pd.Series([np.NaN])])
    name = series.name
    return pd.DataFrame({
       "Task": [name] * len(starts),
       "Type": categories.values,
       "Starts": starts.values,
       "Ends": ends.values})

 
def transform_df_to_start_end(df, index_col=None):
    '''
    Transform DataFrames to start end.
    '''
    index_col_name = index_col
    if index_col is None:
        index_col = df.index.to_series()
    elif isinstance(index_col, str):
        index_col = df[index_col]
    elif isinstance(index_col, pd.Series):
        index_col_name = index_col.name
    else:
        raise
    starts_ends = {}
    for col in df.columns.difference([index_col_name]):
        starts_ends[col] = binary_encode_pd_series(df[col])
    gantt_dfs = []
    for col in starts_ends:
        gantt_df = series_to_gantt_format(df[col], index_col, starts_ends[col])
        gantt_dfs.append(gantt_df)
    return pd.concat(gantt_dfs)



# -*- coding: utf-8 -*-

"""Top-level package for PyGantt."""

__author__ = """PyGantt Developers"""
__email__ = 'seth.mridul@gmail.com'
__version__ = '0.1.0'


from pygantt.pygantt import *
from pygantt.util import *
